/*******************************************************************************
* SiRF_KillStaicNav:   NMEA 0183 & SiRF Binary Protocol library for Arduino    *
*                      example application: Disables the "Static Navigation"   *
*                      feature on Parallax PMB-648 GPS SiRF GPS Receivers.     * 
* Jacob Dilles <jdilles@gmu.edu>   November  2012                              *
********************************************************************************/  

/* Import the SiRF Library */
#include <sirf.h>


/* 
    Declare our GPS Object 
    
    For this example, we assume there is a Parallax PMB-648 connected to
    an Arduino Mega board, as follows:
        Red     ->  +5V
        Black   ->  GND
        Yellow  ->  RX1
        Blue    ->  TX1
*/
SiRF gps;

void setup(){
    /* Hardware Serial for Debugging */
    Serial.begin(9600);
    
    /* Configure the SiRF object on HardwareSerial1, 4800 baud */
    gps.begin(&Serial1, 4800);
   
    /* Give ArduinoIDE a chance to reprogram before spamming Serial */
    Serial.println("Starting...");
    delay(5000);
    Serial.println("Started...");
    
    
    /* 
        Drop to binary mode. Ref: SiRF GPS Protocol Reference Manual Rev 1.30
            -> See "1.2.3 SetSerialPort" on pg 9 
    */
    Serial.print("\nEntering Binary Mode...");
    if(gps.sendNMEA("$PSRF100,0,4800,8,1,0*")){
        Serial.println("Error sending NMEA command");
        return;
    }
    while(gps.receive() != BINARY) Serial.print('.');
    
    
    
    /* 
        Turn off static nav. Ref: SiRF Binary Protocol Reference Manual Rev2.4
            -> See "Static Navigation – Message ID 143" pg 2-16 
    */
    Serial.print("\nTurning off Static Nav (143d)...");
    if(gps.sendHEX("8F00")){
        Serial.println("Error sending binary command 143"); 
        return;
    }
    while(gps.receive() != BINARY) Serial.print('.');
    
    
    
    /* 
        Return to NMEA mode. 
          -> See "Switch To NMEA Protocol – Message ID 129" pg 2-6
    */
    Serial.print("\nReturning to NMEA mode...");
    if(gps.sendHEX("8102010100000101050101010000000100010001000012C0")){
        Serial.println("Error sending binary command 129"); 
        return;
    }
    while(gps.receive() != BINARY) Serial.print('.');
    Serial.println("\nDone! Static Nav should be off");
    
}

/** Just print out the GPRMC sentence and demonstare parsing of params */
void loop(){  
  GPRMC d;
  mode_t mode = gps.receive();
  
  if(mode  == NMEA){
    Serial.println(gps.sentence());
    if(gps.parseGPRMC(&d)){
      Serial.print("Status=");
      Serial.print(d.status);
      Serial.print(", lat=");
      Serial.print(d.lat, 6);
      Serial.print(", lon=");
      Serial.print(d.lon, 6);
      
// Other struct members
//  float     d.speed
//  float     d.angle
//  char      d.utc_year
//  char      d.utc_month
//  char      d.utc_day
//  char      d.utc_hour
//  char      d.utc_min
//  char      d.utc_sec

    }
  }
  
}
